/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/*
 * This file is part of the libetonyek project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */

#include "IWORKProperties.h"

namespace libetonyek
{

IWORK_IMPLEMENT_PROPERTY(Alignment);
IWORK_IMPLEMENT_PROPERTY(Baseline);
IWORK_IMPLEMENT_PROPERTY(BaselineShift);
IWORK_IMPLEMENT_PROPERTY(Bold);
IWORK_IMPLEMENT_PROPERTY(BottomBorder);
IWORK_IMPLEMENT_PROPERTY(Capitalization);
IWORK_IMPLEMENT_PROPERTY(Columns);
IWORK_IMPLEMENT_PROPERTY(Fill);
IWORK_IMPLEMENT_PROPERTY(FirstLineIndent);
IWORK_IMPLEMENT_PROPERTY(FollowingLayoutStyle);
IWORK_IMPLEMENT_PROPERTY(FollowingParagraphStyle);
IWORK_IMPLEMENT_PROPERTY(FontColor);
IWORK_IMPLEMENT_PROPERTY(FontName);
IWORK_IMPLEMENT_PROPERTY(FontSize);
IWORK_IMPLEMENT_PROPERTY(Geometry);
IWORK_IMPLEMENT_PROPERTY(Hyphenate);
IWORK_IMPLEMENT_PROPERTY(Italic);
IWORK_IMPLEMENT_PROPERTY(KeepLinesTogether);
IWORK_IMPLEMENT_PROPERTY(KeepWithNext);
IWORK_IMPLEMENT_PROPERTY(Language);
IWORK_IMPLEMENT_PROPERTY(LayoutMargins);
IWORK_IMPLEMENT_PROPERTY(LayoutParagraphStyle);
IWORK_IMPLEMENT_PROPERTY(LeftBorder);
IWORK_IMPLEMENT_PROPERTY(LeftIndent);
IWORK_IMPLEMENT_PROPERTY(LineSpacing);
IWORK_IMPLEMENT_PROPERTY(ListLabelGeometry);
IWORK_IMPLEMENT_PROPERTY(ListLabelIndent);
IWORK_IMPLEMENT_PROPERTY(ListLabelTypeInfo);
IWORK_IMPLEMENT_PROPERTY(ListLevelStyles);
IWORK_IMPLEMENT_PROPERTY(ListTextIndent);
IWORK_IMPLEMENT_PROPERTY(Opacity);
IWORK_IMPLEMENT_PROPERTY(Outline);
IWORK_IMPLEMENT_PROPERTY(Padding);
IWORK_IMPLEMENT_PROPERTY(PageBreakBefore);
IWORK_IMPLEMENT_PROPERTY(ParagraphBorderType);
IWORK_IMPLEMENT_PROPERTY(ParagraphFill);
IWORK_IMPLEMENT_PROPERTY(ParagraphStroke);
IWORK_IMPLEMENT_PROPERTY(RightBorder);
IWORK_IMPLEMENT_PROPERTY(RightIndent);
IWORK_IMPLEMENT_PROPERTY(SFTAutoResizeProperty);
IWORK_IMPLEMENT_PROPERTY(SFTCellStylePropertyDateTimeFormat);
IWORK_IMPLEMENT_PROPERTY(SFTCellStylePropertyDurationFormat);
IWORK_IMPLEMENT_PROPERTY(SFTCellStylePropertyNumberFormat);
IWORK_IMPLEMENT_PROPERTY(SFTCellStylePropertyLayoutStyle);
IWORK_IMPLEMENT_PROPERTY(SFTCellStylePropertyParagraphStyle);
IWORK_IMPLEMENT_PROPERTY(SFTDefaultBodyCellStyleProperty);
IWORK_IMPLEMENT_PROPERTY(SFTDefaultFooterRowCellStyleProperty);
IWORK_IMPLEMENT_PROPERTY(SFTDefaultHeaderColumnCellStyleProperty);
IWORK_IMPLEMENT_PROPERTY(SFTDefaultHeaderRowCellStyleProperty);
IWORK_IMPLEMENT_PROPERTY(SFTHeaderColumnRepeatsProperty);
IWORK_IMPLEMENT_PROPERTY(SFTHeaderRowRepeatsProperty);
IWORK_IMPLEMENT_PROPERTY(SFTStrokeProperty);
IWORK_IMPLEMENT_PROPERTY(SFTTableBandedCellFillProperty);
IWORK_IMPLEMENT_PROPERTY(SFTTableBandedRowsProperty);
IWORK_IMPLEMENT_PROPERTY(Shadow);
IWORK_IMPLEMENT_PROPERTY(SpaceAfter);
IWORK_IMPLEMENT_PROPERTY(SpaceBefore);
IWORK_IMPLEMENT_PROPERTY(Strikethru);
IWORK_IMPLEMENT_PROPERTY(Stroke);
IWORK_IMPLEMENT_PROPERTY(Tabs);
IWORK_IMPLEMENT_PROPERTY(TextBackground);
IWORK_IMPLEMENT_PROPERTY(TopBorder);
IWORK_IMPLEMENT_PROPERTY(Tracking);
IWORK_IMPLEMENT_PROPERTY(Underline);
IWORK_IMPLEMENT_PROPERTY(VerticalAlignment);
IWORK_IMPLEMENT_PROPERTY(WidowControl);

}

/* vim:set shiftwidth=2 softtabstop=2 expandtab: */
